#include "joystick.h"

#define WAITFAM      Delay10TCYx(5)   // 6us
#define WAITFAM_HALF Delay10TCYx(2)
#define FC_LATCH (PORTCbits.RC0)
#define FC_DATA  (PORTCbits.RC1)
#define FC_CLOCK (PORTCbits.RC2)
#define FC_LATCH_L_CLOCK_H  LATC = 0x04
#define FC_LATCH_H_CLOCK_H  LATC = 0x05
#define FC_LATCH_L_CLOCK_L  LATC = 0x00


// Prottype　definition
void PrepareForReadFC(void);

/// -------------------------------------------------------
/// 読み取り準備（ポート設定など）
/// -------------------------------------------------------
void PrepareForReadFC(void)
{
    // port IN/OUT setup for Famicom
    TRISC = 0b11111010;

    // Prepare for read
    FC_LATCH_L_CLOCK_H;
    WAIT_PORTSET;
}

void ReadPhaseDataFC(unsigned char *phaseData)
{
    // LATCH H->L
    FC_LATCH_H_CLOCK_H;
    WAITFAM;
    WAITFAM;
    FC_LATCH_L_CLOCK_H;
    WAITFAM;

    for(int i=0; i<24; i=i+1)
    {
        //FC_CLOCK = 0;  // CLOCK = L
        FC_LATCH_L_CLOCK_L;
        WAITFAM;
        phaseData[i] = FC_DATA;

        //FC_CLOCK = 1;  // CLOCK = H
        FC_LATCH_L_CLOCK_H;
        WAITFAM;
    }
}

/// -------------------------------------------------------
/// 読み取り＆ジョイスティック出力データ作成
/// （ファイティングパッド6B）
/// -------------------------------------------------------
bool ReadFamicomJoycard
    (char *devType, /*uint32_t*/int *buttons/*, char *hat, char *x, char *y, char *z, char *r */)
{
    unsigned char phaseData[24];
    static unsigned int counter_Failed = 0;
    bool isFamicom = false;

    if(*devType == DEVTYPE_UNKNOWN)
    {
        PrepareForReadFC();
    }


    // If DATA==L when LATCH=L, Famicom Controller connected.
    if(FC_DATA == 0)
    {
        isFamicom = true;
    }

    // Read
    ReadPhaseDataFC(phaseData);


    // Connection detection
    if(isFamicom)
    {
        counter_Failed = 0;

        *devType = DEVTYPE_FAMICOM;

        //Buttons
        *buttons = 0x00000000 |
            ((phaseData[ 1]) ? 0 : BTNBIT_01) |  // B
            ((phaseData[ 0]) ? 0 : BTNBIT_02) |  // A
            ((phaseData[ 2]) ? 0 : BTNBIT_09) |  // Select
            ((phaseData[ 3]) ? 0 : BTNBIT_10) |  // Start
            ((phaseData[ 8]) ? 0 : BTNBIT_03) |  // (SFC A  -> 03)
            ((phaseData[ 9]) ? 0 : BTNBIT_04) |  // (SFC X  -> 04)
            ((phaseData[10]) ? 0 : BTNBIT_05) |  // (SFC Lt -> 05)
            ((phaseData[11]) ? 0 : BTNBIT_06) ;  // (SFC Rt -> 06)

        // Home
        if(
            // A+B+Sel+Start-> Home
            (*buttons & BTNBIT_01) &&
            (*buttons & BTNBIT_02) &&
            (*buttons & BTNBIT_09) &&
            (*buttons & BTNBIT_10)
        )
        {
            *buttons = (*buttons | BTNBIT_13) &
            (~(BTNBIT_01 |
               BTNBIT_02 |
               BTNBIT_09 |
               BTNBIT_10 ));
        }

        Set_AnalogAndHatValue(
            (phaseData[7]),     // R
            (phaseData[6]),     // L
            (phaseData[4]),     // U
            (phaseData[5])      // D
            /*,hat, x, y, z, r*/);

    }
    else
    {
        counter_Failed++;
        if(counter_Failed > 10)
        {
            counter_Failed = 0;
            *devType = DEVTYPE_UNKNOWN;
        }
    }

    return isFamicom;

}