#include "joystick.h"


#define PS_I_DAT            PORTCbits.RC0
#define PS_I_ACK            PORTCbits.RC4
#define PS_CMD              LATCbits.LATC1
#define PS_SEL              LATCbits.LATC2
#define PS_CLK              LATCbits.LATC3
/*
#define PS_CMD              PORTCbits.RC1
#define PS_SEL              PORTCbits.RC2
#define PS_CLK              PORTCbits.RC3
*/
#define WAIT_PS_CLOCKPERIOD_HALF Delay10TCYx(3)
/*Delay10TCYx(8)*/

// Prottype　definition
void PrepareForReadPS(void);
bool InitAndDetectPS(void);
void PS_SendCmd(char *cmd, char *response, int *respLen);
void _PS_WriteRead1byte(char wr, char* rd);

/// -------------------------------------------------------
/// 読み取り準備（ポート設定など）
/// -------------------------------------------------------
void PrepareForReadPS(void)
{
    TRISC = 0b11110001;         // bit0=in, bit1-3=out, bit4=in
    PS_CMD = 1;
    PS_SEL = 1;
    PS_CLK = 1;
    WAIT_PORTSET;
}

/// -------------------------------------------------------
/// 読み取り＆ジョイスティック出力データ作成
/// （プレステパッド）
/// -------------------------------------------------------
bool ReadPS
    (char *devType, int *buttons)
{
    //bool retVal = true;
    //unsigned char phaseData[8];
    //static unsigned int counter_Failed = 0;
    static bool isConnected = false;
    char ID = 0;
    char Stat = 0;
    char cmd[10];
    char resp[10];
    int  respLen;
    int  idx=0;

    if(*devType == DEVTYPE_UNKNOWN)
    {
        PrepareForReadPS();
        isConnected = false;
    }

    // Init
    if(!isConnected)
    {
        isConnected = InitAndDetectPS();
    }

    // Read
    if(isConnected)
    {

        // Start!
        PS_SEL = 0;	// SEL=L
        WAIT_PS_CLOCKPERIOD_HALF;
        WAIT_PS_CLOCKPERIOD_HALF;		// デカいアナログスティックのＩＤが乱れる

        // リードデータ(READ_DATA)
        idx=0;
        cmd[idx++]=0x01;
        cmd[idx++]=0x42;
        cmd[idx++]=0x00;
        cmd[idx++]=0x00;
        cmd[idx++]=0x00;
        cmd[idx++]=0x00;
        cmd[idx++]=0x00;
        cmd[idx++]=0x00;
        cmd[idx++]=0x00;
        PS_SendCmd(cmd, resp, &respLen);

        PS_SEL = 1;	// SEL=H

        //printf("%02x %02x %02x %02x %02x %02x %02x %02x %02x\r\n", resp[0], resp[1], resp[2], resp[3], resp[4], resp[5], resp[6], resp[7], resp[8]);

        // Get ID
        ID = resp[1];
        Stat = resp[2];

        if( !((ID==0xff) && (Stat==0xff)) )//&& !((ID==0x00) && (Stat==0x00)) )
        {
            *buttons = 0x00000000 |
                ((resp[4] & 0x80) ? 0 : BTNBIT_01) |  // □
                ((resp[4] & 0x40) ? 0 : BTNBIT_02) |  // X
                ((resp[4] & 0x20) ? 0 : BTNBIT_03) |  // O
                ((resp[4] & 0x10) ? 0 : BTNBIT_04) |  // △
                ((resp[4] & 0x04) ? 0 : BTNBIT_05) |  // L1
                ((resp[4] & 0x08) ? 0 : BTNBIT_06) |  // R1
                ((resp[4] & 0x01) ? 0 : BTNBIT_07) |  // L2
                ((resp[4] & 0x02) ? 0 : BTNBIT_08) |  // R2
                ((resp[3] & 0x01) ? 0 : BTNBIT_09) |  // Select
                ((resp[3] & 0x08) ? 0 : BTNBIT_10) |  // Start
                ((resp[3] & 0x02) ? 0 : BTNBIT_11) |  // L3
                ((resp[3] & 0x04) ? 0 : BTNBIT_12) ;  // R3

            // Home
            if(
                // Start+Select -> Home
                (*buttons & BTNBIT_09) &&
                (*buttons & BTNBIT_10)
            )
            {
                *buttons = (*buttons | BTNBIT_13) &
                (~(BTNBIT_09 |
                   BTNBIT_10 ));
            }



            // デジタルパッドかどうかで分岐
            if(ID == 0x41)
            {
                // デジタルパッドである

                //Analog sticks
                //   片手コントローラなど、ID=0x41のときはアナログスティックを中立化する
                joystick_input.val[3] = 0x7f;   // X
                joystick_input.val[4] = 0x7f;   // Y
                joystick_input.val[5] = 0x7f;   // Z
                joystick_input.val[6] = 0x7f;   // Rz

                Set_AnalogAndHatValue(
                        (resp[3] & 0x20),
                        (resp[3] & 0x80),
                        (resp[3] & 0x10),
                        (resp[3] & 0x40)
                    /*,hat, x, y, z, r*/);

            }
            else
            {
                // アナログパッドである

                //Hat switch
                joystick_input.val[2] =
                    Get_HatSwitchVal(
                        (resp[3] & 0x20),
                        (resp[3] & 0x80),
                        (resp[3] & 0x10),
                        (resp[3] & 0x40) );

                //Analog sticks
                joystick_input.val[3] = resp[7];   // X
                joystick_input.val[4] = resp[8];   // Y
                joystick_input.val[5] = resp[5];   // Z
                joystick_input.val[6] = resp[6];   // Rz

            }


        }
        else
        {
            //printf("Cannot detect PS Pad... ID:0x%02x, Stat:0x%02x \r\n", ID, Stat);
            isConnected = 0;
        }

        // End!
        //wait_us(CLKSTATETIME__MICROSEC);
        //NiseKabuto::WaitUs(CLKSTATETIME__MICROSEC);
    }

    if(isConnected)
    {
        *devType = DEVTYPE_PS;
    }
    else
    {
        *devType = DEVTYPE_UNKNOWN;
    }

    return isConnected;
}


/// -------------------------------------------------------
/// 初期化と接続判定
/// （プレステパッド）
/// -------------------------------------------------------
bool InitAndDetectPS(void)
{
    char ID = 0;
    char Stat = 0;
    char cmd[10];
    char resp[10];
    int  respLen;
    int idx;
    char status = 1;

    bool _PadIsConnected = true;
    //DBG("In_PS::InitJoyStick(): start!\r\n");

    // 最初はつながってる想定
    _PadIsConnected = true;

    // Start!
    PS_SEL = 0;	// SEL=L
    WAIT_PS_CLOCKPERIOD_HALF;
    WAIT_PS_CLOCKPERIOD_HALF;

    // リードデータ(READ_DATA)
    idx=0;
    cmd[idx++]=0x01;
    cmd[idx++]=0x42;
    cmd[idx++]=0x00;
    cmd[idx++]=0x00;
    cmd[idx++]=0x00;
    cmd[idx++]=0x00;
    cmd[idx++]=0x00;
    cmd[idx++]=0x00;
    cmd[idx++]=0x00;
    PS_SendCmd(cmd, resp, &respLen);

    //DBG("Response of READ_DATA:\r\n");
    //DBG("  %02x %02x %02x %02x %02x %02x %02x %02x %02x\r\n", resp[0], resp[1], resp[2], resp[3], resp[4], resp[5], resp[6], resp[7], resp[8]);

    // Get ID&Stat
    ID = resp[1];
    Stat = resp[2];

    if( !((ID==0xff) && (Stat==0xff)) )//&& !((ID==0x00) && (Stat==0x00)) )
    {
        PS_SEL = 1;	// SEL=H

        WAIT_PS_CLOCKPERIOD_HALF;
        WAIT_PS_CLOCKPERIOD_HALF;

        PS_SEL = 0;	// SEL=L

        // ---------- 1 -------------
        if(status)
        {
            idx=0;
            cmd[idx++]=0x01; cmd[idx++]=0x43; cmd[idx++]=0x00; cmd[idx++]=0x01;
            cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            PS_SendCmd(cmd, resp, &respLen);

            PS_SEL = 1;	// SEL=H

            WAIT_PS_CLOCKPERIOD_HALF;

            PS_SEL = 0;	// SEL=L
        }

        // ---------- 2 -------------
        if(status)
        {
            idx=0;
            cmd[idx++]=0x01; cmd[idx++]=0x45; cmd[idx++]=0x00; cmd[idx++]=0x00;
            cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            PS_SendCmd(cmd, resp, &respLen);

            PS_SEL = 1;	// SEL=H

            WAIT_PS_CLOCKPERIOD_HALF;

            PS_SEL = 0;	// SEL=L
        }

        // ---------- 3 -------------
        if(status)
        {
            idx=0;
            cmd[idx++]=0x01; cmd[idx++]=0x44; cmd[idx++]=0x00; cmd[idx++]=0x01;
            cmd[idx++]=0x04; cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            PS_SendCmd(cmd, resp, &respLen);

            PS_SEL = 1;	// SEL=H

            WAIT_PS_CLOCKPERIOD_HALF;

            PS_SEL = 0;	// SEL=L
        }

        // ---------- 4 -------------
        if(status)
        {
            idx=0;
            cmd[idx++]=0x01; cmd[idx++]=0x45; cmd[idx++]=0x00; cmd[idx++]=0x00;
            cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            PS_SendCmd(cmd, resp, &respLen);

            PS_SEL = 1;	// SEL=H

            WAIT_PS_CLOCKPERIOD_HALF;

            PS_SEL = 0;	// SEL=L
        }

        // ---------- 5 -------------
        if(status)
        {
            idx=0;
            cmd[idx++]=0x01; cmd[idx++]=0x43; cmd[idx++]=0x00; cmd[idx++]=0x00;
            cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            PS_SendCmd(cmd, resp, &respLen);

            PS_SEL = 1;	// SEL=H

            WAIT_PS_CLOCKPERIOD_HALF;

            PS_SEL = 0;	// SEL=L
        }

        // ---------- 6 -------------
        if(status)
        {
            idx=0;
            cmd[idx++]=0x01; cmd[idx++]=0x42; cmd[idx++]=0x00; cmd[idx++]=0x00;
            cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00; cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            PS_SendCmd(cmd, resp, &respLen);

            PS_SEL = 1;	// SEL=H

            WAIT_PS_CLOCKPERIOD_HALF;

            PS_SEL = 0;	// SEL=L
        }

/*
        // ---------------------------------------------
        // CONFIG_MODE_ENTERコマンド発行
        // ---------------------------------------------

        if(status)
        {
            // コンフィギュレーションモードエンター(CONFIG_MODE_ENTER)
            idx=0;
            cmd[idx++]=0x01;
            cmd[idx++]=0x43;
            cmd[idx++]=0x00;
            cmd[idx++]=0x01;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            PS_SendCmd(cmd, resp, &respLen);

            //DBG("Response of CONFIG_MODE_ENTER:\r\n");
            //DBG("  %02x %02x %02x %02x %02x %02x %02x %02x %02x\r\n", resp[0], resp[1], resp[2], resp[3], resp[4], resp[5], resp[6], resp[7], resp[8]);

            PS_SEL = 1;	// SEL=H

            WAIT_PS_CLOCKPERIOD_HALF;

            PS_SEL = 0;	// SEL=L
        }

        // ---------------------------------------------
        // QUERY_MODEL_AND_MODEコマンド発行
        // ---------------------------------------------

        if(status)
        {
            // クエリーモデル＆モード(QUERY_MODEL_AND_MODE)
            idx=0;
            cmd[idx++]=0x01;
            cmd[idx++]=0x45;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            PS_SendCmd(cmd, resp, &respLen);

            //DBG("Response of QUERY_MODEL_AND_MODE:\r\n");
            //DBG("  %02x %02x %02x %02x %02x %02x %02x %02x %02x\r\n", resp[0], resp[1], resp[2], resp[3], resp[4], resp[5], resp[6], resp[7], resp[8]);
            if(resp[6]==2)
            {
                //DBG("    Dual Shock is connected. Mode:%d (01:Analog, 00:Digital)\r\n", resp[5]);
            }
            else if(resp[6]==1)
            {
                //DBG("    JogCon is connected. Mode:%d (01:Analog, 00:Digital)\r\n", resp[5]);
            }

            PS_SEL = 1;	// SEL=H

            WAIT_PS_CLOCKPERIOD_HALF;

            PS_SEL = 0;	// SEL=L
        }

        // ---------------------------------------------
        // SET_MODE_AND_LOCKコマンド発行
        // ---------------------------------------------

        if(status)
        {
            // セットモード＆ロック(SET_MODE_AND_LOCK
            idx=0;
            cmd[idx++]=0x01;
            cmd[idx++]=0x44;
            cmd[idx++]=0x00;
            cmd[idx++]=0x01;
            cmd[idx++]=0x04;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            PS_SendCmd(cmd, resp, &respLen);

            //DBG("Response of SET_MODE_AND_LOCK:\r\n");
            //DBG("  %02x %02x %02x %02x %02x %02x %02x %02x %02x\r\n", resp[0], resp[1], resp[2], resp[3], resp[4], resp[5], resp[6], resp[7], resp[8]);

            PS_SEL = 1;	// SEL=H

            WAIT_PS_CLOCKPERIOD_HALF;

            PS_SEL = 0;	// SEL=L
        }

        // ---------------------------------------------
        // 2度目のQUERY_MODEL_AND_MODEコマンド発行
        // ---------------------------------------------

        if(status)
        {
            // クエリーモデル＆モード(QUERY_MODEL_AND_MODE)
            idx=0;
            cmd[idx++]=0x01;
            cmd[idx++]=0x45;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            cmd[idx++]=0x00;
            PS_SendCmd(cmd, resp, &respLen);

            //DBG("Response of QUERY_MODEL_AND_MODE:\r\n");
            //DBG("  %02x %02x %02x %02x %02x %02x %02x %02x %02x\r\n", resp[0], resp[1], resp[2], resp[3], resp[4], resp[5], resp[6], resp[7], resp[8]);
            if(resp[6]==2)
            {
                //DBG("    Dual Shock is connected. Mode:%d (01:Analog, 00:Digital)\r\n", resp[5]);
            }
            else if(resp[6]==1)
            {
                //DBG("    JogCon is connected. Mode:%d (01:Analog, 00:Digital)\r\n", resp[5]);
            }

            PS_SEL = 1;	// SEL=H

            // 旧処理では1フレーム分待ってた
            WAIT_PS_CLOCKPERIOD_HALF;

            PS_SEL = 0;	// SEL=L
        }

        // ---------------------------------------------
        // CONFIG_MODE_EXITコマンド発行
        // ---------------------------------------------

        if(status)
        {
                // コンフィギュレーションモードイクジット(CONFIG_MODE_EXIT)
                idx=0;
                cmd[idx++]=0x01;
                cmd[idx++]=0x43;
                cmd[idx++]=0x00;
                cmd[idx++]=0x00;
                cmd[idx++]=0x00;
                cmd[idx++]=0x00;
                cmd[idx++]=0x00;
                cmd[idx++]=0x00;
                cmd[idx++]=0x00;
                PS_SendCmd(cmd, resp, &respLen);

                //DBG("Response of CONFIG_MODE_EXIT:\r\n");
                //DBG("  %02x %02x %02x %02x %02x %02x %02x %02x %02x\r\n", resp[0], resp[1], resp[2], resp[3], resp[4], resp[5], resp[6], resp[7], resp[8]);
        }
 */
    }
    else
    {
        //printf("Cannot detect PS Pad... ID:0x%02x, Stat:0x%02x \r\n", ID, Stat);
        PS_SEL = 1;	// SEL=H
        _PadIsConnected = false;
    }


    // End!
    PS_SEL = 1;	// SEL=H
    WAIT_PS_CLOCKPERIOD_HALF;


    //DBG("In_PS::InitJoyStick(): end!\r\n");

    return _PadIsConnected;
}

//
// Send Command
//		cmdLenの長さのコマンドcmdを送り、
//		responseの長さもrespLenに入れて返す
//	呼び出し元で
//		- cmdLenは正しい値を入れること
//		- responseは確保しておくこと
void PS_SendCmd(char *cmd, char *response, int *respLen)
{
    int cmdIdx  = 0;
    int respIdx = 0;
    int loopMax = 10000;
    int loopCount;

    // Start!
    // 5回繰り返す
    // ID -> response[1]
    // Stat -> response[2]

    for(int i=0; i<5; i++)
    {
        _PS_WriteRead1byte(cmd[cmdIdx++], &(response[respIdx++]));

        if(i!=4)
        {
            loopCount=0;
            while(loopCount<20)
            {
                loopCount++;
            }
            /*
            loopCount=0;
            while(PS_I_ACK == 1)
            {
                loopCount++;
                if(loopCount>loopMax)
                {
                    break;
                }
            }
             */

        //    WAIT_PS_CLOCKPERIOD_HALF;
        }
    }

    // アナログの場合に応答長さが変わる場合
    if( (response[1]&0x0f) == 0x03 )
    {
        /*
        loopCount=0;
        while(PS_I_ACK == 1)
        {
            loopCount++;
            if(loopCount>loopMax)
            {
                    break;
            }
        }
        */
        loopCount=0;
        while(loopCount<20)
        {
            loopCount++;
        }


        // WAIT_PS_CLOCKPERIOD_HALF;

        // あと４回繰り返し
        for(int i=0; i<4; i++)
        {
            _PS_WriteRead1byte(cmd[cmdIdx++], &(response[respIdx++]));

            if(i!=4)
            {
                loopCount=0;
                while(loopCount<20)
                {
                    loopCount++;
                }

                /*
                loopCount=0;
                while(PS_I_ACK == 1)
                {
                    loopCount++;
                    if(loopCount>loopMax)
                    {
                        break;
                    }
                }
                 */

                //WAIT_PS_CLOCKPERIOD_HALF;
            }
        }
    }
    *respLen = respIdx;
}

/// -------------------------------------------------------
/// Write 1byte & Read 1byte
/// -------------------------------------------------------
void _PS_WriteRead1byte(char wr, char* rd)
{
	//pin setting
	PS_CMD = 1;   // output CMD = H
	PS_CLK = 1;   // output CLK = H

	// init
	*rd=0;

	// Start!

	// 8 bit
	for(int i=0; i<8; i++)
	{
		// 引用
		// データ／コマンドはクロックの立ち下がりで出力.立ち上がりで取り込みとする.
		// データ／コマンドはクロックの立ち上がりに確定していること.

		PS_CMD = ((wr>>i)&1) ? 1 : 0;
		PS_CLK = 0;   // output CLK = L
		WAIT_PS_CLOCKPERIOD_HALF;

		PS_CLK = 1;   // output CLK = H

		*rd = *rd | ((PS_I_DAT & 1)<<i);
	}

	WAIT_PS_CLOCKPERIOD_HALF;

}


